@@ -179,37 +179,6 @@ $$ language plpgsql;
             batch_item,
         )
 
-    # Update a batch item in the batch log, making sure to fail if the id and updated_at don't match, providing a guarantee of atomic operation
-    # Returns updated_at, so caller can check if it was updated or not (compare it to argument)
-    def update_batch_item(self, batch_item):
-        return self.dbc.query_one(
-            """
-                update
-                    batch_log
-                set
-                      priority = %(priority)s
-                    , ttl_seconds = %(ttl_seconds)s
-                    , data = %(data)s
-                    , result = %(result)s
-                    , type = %(type)s
-                    , status = %(status)s
-                    , throttle_key = %(throttle_key)s
-                    , throttle_limit = %(throttle_limit)s
-                    , throttle_period = %(throttle_period)s
-                    , source = %(source)s
-                    , error = %(error)s
-                    , updated_at = now()
-                where
-                    id = %(id)s
-                and
-                    updated_at = %(updated_at)s
-                returning
-                    updated_at
-                ;
-                """,
-            batch_item,
-        )
-
     # Update status of all "in progress" jobs that already spent more than ttl time to "hung"
     def update_batch_hang_status(self):
         return self.dbc.query_none(
@@ -220,7 +189,6 @@ $$ language plpgsql;
                       status = %(to_status)s
                     , updated_at=  now()
                     , last_finished_at = now()
-                    
                 where
                     status = %(from_status)s
                 and
@@ -231,7 +199,7 @@ $$ language plpgsql;
                     extract('epoch' from ( now() - updated_at )) > ttl_seconds
                 ;
                 """,
-            {"from_status": (self.IN_PROGRESS,), "to_status": (self.HUNG,)},
+            {"from_status": (self.IN_PROGRESS,), "to_status": (self.HUNG,), "in_progress_status": (self.IN_PROGRESS,)},
         )
 
     # Update status of all jobs with from_status to to_status
@@ -241,13 +209,36 @@ $$ language plpgsql;
                 update
                     batch_log
                 set
-                     status = %(to_status)s
+                      last_started_at =
+                        (case
+                            when
+                                b.status is distinct from %(in_progress_status)s
+                            and
+                                %(status)s = %(in_progress_status)s
+                            then
+                                now()
+                            else
+                                b.last_started_at
+                        end)
+
+                    , last_finished_at =
+                        (case
+                            when
+                                b.status = %(in_progress_status)s
+                            and
+                                %(status)s is distinct from %(in_progress_status)s
+                            then
+                                now()
+                            else
+                                b.last_finished_at
+                        end)
+                    , status = %(to_status)s
                     , updated_at = now()
                 where
                     status = %(from_status)s
                 ;
                 """,
-            {"from_status": (from_status,), "to_status": (to_status,)},
+            {"from_status": (from_status,), "to_status": (to_status,), "in_progress_status": (self.IN_PROGRESS,)},
         )
 
     # Update status of a batch item in the batch log, making sure to fail if the id and updated_at don't match, providing a guarantee of atomic operation
@@ -302,7 +293,7 @@ $$ language plpgsql;
                     , updated_at
                 ;
                 """,
-            {"id": (id,), "status": (status,), "error": (error,), "result": (result,), "updated_at": (updated_at,), "in_progress_status": (IN_PROGRESS,)},
+            {"id": (id,), "status": (status,), "error": (error,), "result": (result,), "updated_at": (updated_at,), "in_progress_status": (self.IN_PROGRESS,)},
         )
 
     # Simplified version of bump_batch_item where status is updated, no questions asked
@@ -312,13 +303,35 @@ $$ language plpgsql;
                 update
                     batch_log
                 set
-                      status = %(status)s
+                      last_started_at =
+                        (case
+                            when
+                                b.status is distinct from %(in_progress_status)s
+                            and
+                                %(status)s = %(in_progress_status)s
+                            then
+                                now()
+                            else
+                                b.last_started_at
+                        end)
+                    , last_finished_at =
+                        (case
+                            when
+                                b.status = %(in_progress_status)s
+                            and
+                                %(status)s is distinct from %(in_progress_status)s
+                            then
+                                now()
+                            else
+                                b.last_finished_at
+                        end)
+                    , status = %(status)s
                     , updated_at = now()
                 where
                     id = %(id)s
                 ;
                 """,
-            {"id": (id,), "status": (status,)},
+            {"id": (id,), "status": (status,), "in_progress_status": (self.IN_PROGRESS,)},
         )
 
     # Update a batch item in the batch log, making sure to fail if the id and updated_at don't match, providing a guarantee of atomic operation
@@ -328,7 +341,29 @@ $$ language plpgsql;
                 update
                     batch_log
                 set
-                      status=%(to_status)s
+                    last_started_at =
+                      (case
+                          when
+                              b.status is distinct from %(in_progress_status)s
+                          and
+                              %(status)s = %(in_progress_status)s
+                          then
+                              now()
+                          else
+                              b.last_started_at
+                      end)
+                  , last_finished_at =
+                      (case
+                          when
+                              b.status = %(in_progress_status)s
+                          and
+                              %(status)s is distinct from %(in_progress_status)s
+                          then
+                              now()
+                          else
+                              b.last_finished_at
+                      end)
+                    , status=%(to_status)s
                     , updated_at=now()
                 where
                     status = %(from_status)s
@@ -360,7 +395,7 @@ $$ language plpgsql;
                     , updated_at
                 ;
                 """,
-            {"from_status": (from_status,), "to_status": (to_status,)},
+            {"from_status": (from_status,), "to_status": (to_status,), "in_progress_status": (self.IN_PROGRESS,)},
         )
 
     # Update a batch item in the batch log, making sure to fail if the id and updated_at don't match, providing a guarantee of atomic operation
@@ -371,7 +406,29 @@ $$ language plpgsql;
                 update
                     batch_log
                 set
-                     status=%(to_status)s
+                    last_started_at =
+                      (case
+                          when
+                              b.status is distinct from %(in_progress_status)s
+                          and
+                              %(status)s = %(in_progress_status)s
+                          then
+                              now()
+                          else
+                              b.last_started_at
+                      end)
+                  , last_finished_at =
+                      (case
+                          when
+                              b.status = %(in_progress_status)s
+                          and
+                              %(status)s is distinct from %(in_progress_status)s
+                          then
+                              now()
+                          else
+                              b.last_finished_at
+                      end)
+                    , status=%(to_status)s
                     , updated_at=now()
                 where
                     status = %(from_status)s
@@ -405,7 +462,7 @@ $$ language plpgsql;
                     , updated_at
                 ;
                 """,
-            {"from_status": (from_status,), "to_status": (to_status,)},
+            {"from_status": (from_status,), "to_status": (to_status,), "in_progress_status": (self.IN_PROGRESS,)},
         )
 
     # Delete single item by id
@@ -444,101 +501,6 @@ $$ language plpgsql;
                 """
         )
 
-    # Get batch items from batch log fitting the optional filter
-    def get_batch_items(self, id=None, priority=None, ttl_seconds=None, type=None, status=None, throttle_key=None, source=None, limit=1):
-        return self.dbc.query_many(
-            """
-                select
-                      id
-                    , priority
-                    , ttl_seconds
-                    , data
-                    , result
-                    , type
-                    , status
-                    , throttle_key
-                    , throttle_limit
-                    , throttle_period
-                    , source
-                    , created_at
-                    , updated_at
-                from
-                    batch_log
-                where
-                    true
-                and
-                    (%(id)s is null or id = any(%(id)s))
-                and
-                    (%(priority)s is null or priority = any(%(priority)s))
-                and
-                    (%(ttl_seconds)s is null or ttl_seconds = any(%(ttl_seconds)s))
-                and
-                    (%(type)s is null or type = any(%(type)s))
-                and
-                    (%(status)s is null or status = any(%(status)s))
-                and
-                    (%(throttle_key)s is null or throttle_key = any(%(throttle_key)s))
-                and
-                    (%(source)s is null or source = any(%(source)s))
-                order by
-                      priority desc
-                    , updated_at asc
-                limit
-                    %(limit)s
-                ;
-                """,
-            {"id": (id,), "priority": (priority,), "type": (type,), "status": (status,), "throttle_key": (throttle_key,), "source": (source,), "limit": (limit,)},
-        )
-
-    # Get batch item by id from batch log
-    def get_batch_item_by_id(self, id):
-        return self.dbc.query_one(
-            """
-                select
-                      id
-                    , priority
-                    , ttl_seconds
-                    , data
-                    , result
-                    , type
-                    , status
-                    , throttle_key
-                    , throttle_limit
-                    , throttle_period
-                    , source
-                    , created_at
-                    , updated_at
-                from
-                    batch_log
-                where
-                    id = %(id)s
-                order by
-                      priority desc
-                    , updated_at asc
-                limit
-                    1
-                ;
-                """,
-            {"id": (id,)},
-        )
-
-    # Get distinct batch types+status counts
-    def get_type_status_counts(self):
-        return self.dbc.query_many(
-            """
-                select
-                     count(*) as count
-                    , type as name
-                    , status as status
-                from
-                    batch_log
-                group by
-                      type
-                    , status
-                ;
-                """
-        )
-
     # Get distinct batch types with counts
     def get_type_counts(self):
         return self.dbc.query_many(
@@ -569,34 +531,6 @@ $$ language plpgsql;
                 """
         )
 
-    # Get count of ready pending batch items taking into acount throttling
-    def get_throttled_pending_count(self):
-        return self.dbc.query_many(
-            """
-                select
-                      count(*) as count
-                    , min(gcra_throttle(throttle_key, throttle_limit, throttle_period, false)) as wait_millis
-                from
-                    batch_log
-                ;
-                """
-        )
-
-    # Get ready pending batch items taking into acount throttling
-    def get_throttled_pending(self):
-        return self.dbc.query_many(
-            """
-                select
-                      id
-                    , gcra_throttle(throttle_key, throttle_limit, throttle_period, false) as wait_millis
-                from
-                    batch_log
-                order by
-                    wait_millis asc
-                ;
-                """
-        )
-
     def get_job_counts(self):
         return self.dbc.query_many(
             """
@@ -617,7 +551,7 @@ $$ language plpgsql;
         )
 
     # Get batch items from batch log sorted by last active
-    def get_last_jobs(self, id=None, priority=None, type=None, status=None, throttle_key=None, source=None, error=None, limit=1):
+    def get_jobs(self, id=None, priority=None, ttl_seconds=None, type=None, status=None, throttle_key=None, source=None, error=None, limit=1):
         return self.dbc.query_many(
             """
                 select
@@ -631,9 +565,12 @@ $$ language plpgsql;
                     , throttle_key
                     , throttle_limit
                     , throttle_period
+                    , source
                     , error
                     , updated_at - created_at as runtime
-                    , extract(EPOCH from (updated_at - created_at)) as runtime_ts
+                    , extract(epoch from (updated_at - created_at)) as runtime_ts
+                    , last_started_at
+                    , last_finished_at
                     , updated_at
                     , created_at
                 from
@@ -644,6 +581,8 @@ $$ language plpgsql;
                     (%(id)s is null or id = any(%(id)s))
                 and
                     (%(priority)s is null or priority = any(%(priority)s))
+                and
+                    (%(ttl_seconds)s is null or ttl_seconds = any(%(ttl_seconds)s))
                 and
                     (%(type)s is null or type = any(%(type)s))
                 and
@@ -656,7 +595,8 @@ $$ language plpgsql;
                     (%(source)s is null or source = any(%(source)s))
                 order by
                       updated_at desc
-                    , runtime asc, priority desc
+                    , runtime asc
+                    , priority desc
                 limit
                     %(limit)s
                 ;
@@ -664,6 +604,10 @@ $$ language plpgsql;
             {"id": (id,), "priority": (priority,), "type": (type,), "status": (status,), "throttle_key": (throttle_key,), "source": (source,), "error": (error,), "limit": (limit,)},
         )
 
+    #    , min( extract('epoch' from updated_at - created_at)) as low
+    # , max( extract('epoch' from updated_at - created_at)) as high
+    # , percentile_cont(0.5) within group (order by extract('epoch' from "test_median".updated_at - "test_median".created_at)::float ) as median
+
     def get_now(self):
         r = self.dbc.query_one(
             """
